/*!

	\file		plugin_manager.h
	\brief 	API of the PluginManager
	\author	Aram Verstegen

	This header contains the declarations of the NextGen Library functions:
	-# startPluginManager(): start the PluginManager --> load plugins as dictated by the EvaluationManager
	-# stopPluginManager():	stop the PluginManager
	-# runPlugin(plugin_name): find plugin with name plugin_name in list, and call its run function.
  -# termPlugin(plugin_name): find plugin with name plugin_name in list, and call its term function.

  -# get_running_plugin(): returns a pointer to the node containing the running plugin's DL handle.
  -# set_running_plugin(plugin_handle): set a pointer to the node containing the running plugin's DL handle.

*/
#ifndef PLUGIN_MANAGER_H
#define PLUGIN_MANAGER_H

#include "eef_return_codes.h"
#include "pdl.h"
/* Function pointer type */

extern EES_RC											start_plugin_manager(void);
extern EES_RC											stop_plugin_manager(void);
extern EES_RC                     add_plugin_struct(int argc, char* argv[], char* name);

extern EES_RC                     initialize_plugins(void);
extern EES_RC                     run_plugins(policy_t* policies);
extern EES_RC                     term_plugins(void);

extern eef_plugindl_t *           get_plugin(char*);
extern eef_plugindl_t *           get_running_plugin(void);
extern void                       set_running_plugin(eef_plugindl_t * plugin);

extern char*                      get_running_plugin_name(void);
extern void                       set_running_plugin_name(char* plugin_name);

extern void                       plugin_manager_start_threading(void);

#endif
