#ifndef GRIDMAPDIR_H
#define GRIDMAPDIR_H

#define MATCH_INCLUDE             ((unsigned short)0x0001)
#define MATCH_EXCLUDE             ((unsigned short)0x0000)
#define MATCH_WILD_CHARS          ((unsigned short)0x0002)
#define MATCH_NO_WILD_CHARS       ((unsigned short)0x0000)
#define OVERRIDE_INCONSISTANCY    ((unsigned short)0x0004)
#define MATCH_ONLY_DN             ((unsigned short)0x0008)
#define MATCH_STRICT_PREFIX_NUM   ((unsigned short)0x0010)
#define ONLY_USE_EXISTING_LEASE   ((unsigned short)0x0020)
#define INCLUDE    3
#define EXCLUDE    4

/******************************************************************************
Function:   gridmapdir_otherlink
Description:
	find another link in MAPDIR to the same inode as firstlink
	and change the modification time of firstlink to now (so that we
	always know when this pair was last used)

Parameters:
	gridmapdir, the directory to look for links
	firstlink, the filename of the link we already know

Returns:
	a pointer to the other link's filename (without path) or NULL if none
	found (this is malloc'd and will need freeing)
******************************************************************************/
char * gridmapdir_otherlink(const char * gridmapdir, char * firstlink);

/******************************************************************************
Function:   gridmapdir_urlencode
Description:
	Convert string to URL encoded and return pointer to the encoded
	version, obtained through malloc. Calling routine must free
	this. Here "URL encoded" means anything other than an isalnum()
	goes to %HH where HH is its ascii value in hex; also A-Z => a-z
	This name is suitable for filenames since no / or spaces.
	The part of the poolindex that contains the local group mappings
	and the mapcounter is copied as-is in the encoded URL. The subject DN
	and this latter part is separated by the magic character '\001', which
	is translated in the encoded string as ':'

Parameters:
	rawstring, the string to be converted

Returns:
	a pointer to the encoded string or NULL if the malloc failed
******************************************************************************/
char * gridmapdir_urlencode(const char * rawstring);

/******************************************************************************
Function:   gridmapdir_newlease
Description:
	Search for an unleased local username to give to the globus user
	corresponding to encodedfilename, and then lease it.

Parameters:
	gridmapdir, the directory to look for links
	encodedfilename, URL-encoded globus client name and pathname of
	   the globus user who requested authentication
	usernameprefix, the prefix of acceptable usernames (or "\0")

Returns:
	int
	0 == good
	1 == noleaseavailable (any more) in gridmapdir
	2 == Failure in opening the gridmapdir
	3 == Out of memory
	4 == other failure
******************************************************************************/
int gridmapdir_newlease(const char * gridmapdir, const char * encodedglobusidp, const char * usernameprefix, unsigned short matching_type);

/******************************************************************************
Function:   gridmapdir_userid
Description:
	This is equivalent to globus_gss_assist_gridmap but for the dynamic
	user ids in the gridmapdir: maps a globusID to a local unix user id,
	either one already leased, or calls gridmapdir_newlease() to obtain
	a new lease. This is called by globus_gss_assist_gridmap if the
	local user id in the static gridmap file begins . (for a dynamic id)

Parameters:
	globusidp, globus client name who requested authentication
	usernameprefix, prefix of the local usernames which would be acceptable (or "\0" )
	*userid returned userid name for local system.

Returns:
	0 on success
	!=0 on failure

******************************************************************************/
int gridmapdir_userid(const char * gridmapdir, const char * globusidp, const char * usernameprefix, char ** useridp, unsigned short matching_type);

/******************************************************************************
Function:   gridmapdir_globusid
Description:
	This is equivalent to globus_gss_assist_map_local_user but for the
	dynamic user ids in the gridmapdir: search through leases to find
	which globusID corresponds to a local unix user id.
	This is called by globus_gss_assist_map_local_user

Parameters:
	globus client name who requested authentication
	*userid returned userid name for local system.

Returns:

	0 on success
	!=0 on failure

******************************************************************************/
int gridmapdir_globusid(const char * gridmapdir, char * useridp, char ** globusidp);

/******************************************************************************
Function:   gridmapdir_userok
Description:
	This is equivalent to globus_gss_assist_userok but for the dynamic
	user ids in the gridmapdir: finds the local unix username leased to
	a globusID and compare with the username being checked.
	This is called by globus_gss_assist_userok if the local user id in
	the static gridmap file is -  (for a dynamic id)

Parameters:
	globus client name who requested authentication
	userid to be checked

Returns:
	0 on success (authorization allowed)
	!=0 on failure or authorization denied

******************************************************************************/
int gridmapdir_userok(const char * gridmapdir, char * globusidp, char * userid);

#endif /* GRIDMAPDIR_H */
