%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include "pdl.h"
#include "eef_log.h"

%}

%union {
  record_t* record;
  rule_t*   rule;
};

%token<record> LABEL PORTVAR PORT TERM TRANS EMPTYLINE COMMENT PVAR PATH STRING
%type<record>  config var_list var path policy_list string
%type<rule> policy rule;

%token_table

%nonassoc LABEL

%%

config:         /*  empty  */             { EEF_log(LOG_WARNING, "Config file is empty."); }
              | var_list policy_list
              | policy_list
              | var_list                  { EEF_log(LOG_ERR, "Config file contains no policy rules."); }
;


var_list:       var
              | var_list var
;

var:            PORTVAR '=' PORT    	  { set_pdl_port($3); }
	      |	TERM  '=' string          { add_variable($1, $3); }
              | TERM  '=' TERM            { add_variable($1, $3); }
              | PVAR '=' path '\n'        { set_pdl_path($3);         }
              | PVAR '='                  { EEF_log(LOG_NOTICE, "No value assigned to path. Using %s as path instead.", get_pdl_path()); }
    /*
              | TERM  '='                 { eef_log(LOG_ERR, "No value assigned to %s.", $1->string); }
              | TERM                      { eef_log(LOG_ERR, "Found a non classified term: %s.", $1->string); }
    */
;

string:         STRING                    { $$ = $1;                     }
              | string STRING             { $$ = concat_strings_with_space($1, $2); }
;

path:           PATH                      { $$ = $1;                     }
              | path PATH                 { $$ = concat_strings($1, $2); }
;

policy_list:    LABEL policy              { add_policy($1, $2); }
              | policy_list LABEL policy  { add_policy($2, $3); }
              | policy_list LABEL         { EEF_log(LOG_WARNING, "expecting rule definitions.");
                                            EEF_log(LOG_WARNING, "no rules specified for policy: '%s' at line %d.", $2->string, $2->lineno);
                                            remove_policy($2);
                                          }
              | LABEL                     { EEF_log(LOG_WARNING, "expecting rule definitions.");
                                            EEF_log(LOG_WARNING, "no rules specified for policy: '%s' at line %d.", $1->string, $1->lineno);
                                            remove_policy($1);
                                          }
;

policy:         rule                      { $$ = $1; }
              | policy rule               { $$ = $1; }
;

rule:           TERM                      { $$ = add_rule($1, NULL, NULL); }
              | TERM TRANS TERM           { $$ = add_rule($1, $3,  NULL); }
              | TERM TRANS TERM '|' TERM  { $$ = add_rule($1, $3, $5); }
              | '~' TERM TRANS TERM       { $$ = add_rule($2,  NULL, $4); }
;

%%

void set_yylval(record_t* r) {
  yylval.record = r;
}
